local esCommon = require("ess.common.ic");
local esMergeOptions = require("ess.merger.options");
local mergeAnimate = require("ess.animate.ic"):new();

function mergeAnimate:doPerform()
    local totalStackUses = esCommon.items.getStackUses(self.extra[1]);
    local usesPerItem = esCommon.numbers.round(1 / self.item:getUseDelta());

    for i=0, self.extra[1]:size() - 1 do
        local item = self.extra[1]:get(i);

        if (totalStackUses > usesPerItem) then
            item:setUsedDelta(1);
            totalStackUses = totalStackUses - usesPerItem;
        elseif(totalStackUses > 0) then
            local delta = totalStackUses * item:getUseDelta();
            item:setUsedDelta(delta);
            totalStackUses = 0;
        else
            item:setUsedDelta(0);
            if (not esMergeOptions.getOption("mergeKeepOn")) then
                item:Use();
            end
        end

    end

    if (isClient()) then
        local merged = esCommon.items.getStackItems(self.item:getFullType(), self.character:getInventory());
        esCommon.items.moveTo(merged, self.extra[2], self.character)
    end
end

local function doMenuMerge(item, itemStack, player)
    local animate = mergeAnimate:new(player, item, 80);
    animate:setExtra({ itemStack, item:getContainer() });

    if (isClient()) then
        esCommon.items.moveTo(itemStack, player:getInventory(), player);
    else
        esCommon.items.moveTo(itemStack, item:getContainer(), player);
    end

    ISTimedActionQueue.add(animate);
end

local function doMenuFill(item, itemStack, player)
    local animate = mergeAnimate:new(player, item, 80);
    local usesPerItem = esCommon.numbers.round(1 / item:getUseDelta());
    local fillUses = usesPerItem - item:getDrainableUsesInt();
    local newStack = LuaList:new();

    newStack:add(item);
    for i = 0, itemStack:size() - 1 do
        local localItem = itemStack:get(i);
        if (localItem ~= item) then
            fillUses = fillUses - localItem:getDrainableUsesInt();
            newStack:add(localItem);
            if (fillUses <= 0) then
                break;
            end
        end
    end

    animate:setExtra({ newStack, item:getContainer() });

    if (isClient()) then
        esCommon.items.moveTo(newStack, player:getInventory(), player);
    else
        esCommon.items.moveTo(newStack, item:getContainer(), player);
    end

    ISTimedActionQueue.add(animate);
end

local function filterDrainable(itemStack)
    local drainables = LuaList:new();
    local selectedItem;

    for i = 0, itemStack:size() - 1 do
        local testItem = itemStack:get(i);
        if testItem:IsDrainable() and testItem:getUseDelta() > 0 and not testItem:isFavorite() then

            if (selectedItem == nil) then
                drainables:add(testItem);
                selectedItem = testItem;
            elseif (selectedItem:getName() == testItem:getName()) then
                drainables:add(testItem);
            end

        end
    end

    return drainables, selectedItem;
end

local ISIPCMCreateMenu = ISInventoryPaneContextMenu.createMenu;
ISInventoryPaneContextMenu.createMenu = function(player, isInPlayerInventory, items, x, y, origin)
    local baseContext = ISIPCMCreateMenu(player, isInPlayerInventory, items, x, y, origin);
    if not (esMergeOptions.getOption("mergeOn") and baseContext) then return baseContext end;

    local thisStack, selectedItem = filterDrainable(esCommon.items.getStackFromSelection(items));
    if thisStack:size() == 0 then return baseContext end;

    local allItems = esCommon.items.getStackItems(selectedItem:getFullType(), esCommon.containers.getAll(player))

    if (allItems:size() > 1) then
        local recipeName = getText("UI_mo_InstantConsolidate_FILL_THIS") .. " " .. selectedItem:getDisplayName();
        baseContext:addOption(recipeName, selectedItem, doMenuFill, allItems, esCommon.player.getPlayerObject(player));
    end
    if (thisStack:size() > 1) then
        local recipeName = getText("UI_mo_InstantConsolidate_MERGE_THIS") .. " " .. selectedItem:getDisplayName() .. " (" .. thisStack:size() .. ")";
        baseContext:addOption(recipeName, selectedItem, doMenuMerge, thisStack, esCommon.player.getPlayerObject(player));
    end
    if (allItems:size() > 1) then
        local recipeName = getText("UI_ESIC_CONSOLIDATE_THIS_ITEM") .. " " .. thisStack:get(0):getDisplayName() .. " (" .. allItems:size() .. ")";
        baseContext:addOption(recipeName, selectedItem, doMenuMerge, allItems, esCommon.player.getPlayerObject(player));
    end

    return baseContext;
end

local baseISInventoryPane = ISInventoryPane.drawItemDetails;
function ISInventoryPane:drawItemDetails(item, y, xoff, yoff, red)

    if (esMergeOptions.getOption("mergeOn") and item ~= nil and instanceof(item, "Drainable") and item:getUseDelta() > 0) then
        local maxUses = esCommon.numbers.round(1 / item:getUseDelta());
        local usesLeft = item:getDrainableUsesInt();

        local hdrHgt = self.headerHgt
        local top = hdrHgt + y * self.itemHgt + yoff
        local fgBar = {r=0.0, g=0.6, b=0.0, a=0.7}
        local fgText = {r=0.6, g=0.8, b=0.5, a=0.6}
        local text = getText("IGUI_invpanel_Remaining") .. ": "..usesLeft.."/"..maxUses;
        self:drawTextAndProgressBar(text, item:getUsedDelta(), xoff, top, fgText, fgBar)
    else
        baseISInventoryPane(self,item, y, xoff, yoff, red);
    end

end
